with GNATCOLL.Symbols;  use GNATCOLL;
with GNATCOLL.Utils;
with SPARK_Util;
with Sinput;            use Sinput;
with Common_Containers; use Common_Containers;
with Why.Sinfo;         use Why.Sinfo;

package body Why.Atree.To_Json is

   ---------------------
   --  General types  --
   ---------------------

   function Boolean_To_Json (B : Boolean) return JSON_Value;
   function Uint_To_Json (I : Uint) return JSON_Value;
   function Ureal_To_Json (I : Ureal) return JSON_Value;
   function String_Sets_Set_To_Json (S : String_Sets.Set) return JSON_Value;

   function Boolean_To_Json (B : Boolean) return JSON_Value is (Create (B));

   function Uint_To_Json (I : Uint) return JSON_Value is
     (Create (UI_Image (I, Decimal)));

   function Ureal_To_Json (I : Ureal) return JSON_Value is
      Res : constant JSON_Value := Create (Empty_Array);
   begin
      Append (Res, Uint_To_Json (Numerator (I)));
      Append (Res, Uint_To_Json (Denominator (I)));
      Append (Res, Create (Integer (Rbase (I))));
      Append (Res, Create (UR_Is_Negative (I)));
      return Res;
   end Ureal_To_Json;

   function String_Sets_Set_To_Json (S : String_Sets.Set) return JSON_Value is
      Res : constant JSON_Value := Create (Empty_Array);
   begin
      for El of S loop
         Append (Res, Create (El));
      end loop;
      return Res;
   end String_Sets_Set_To_Json;

   ------------------
   --  Gnat types  --
   ------------------

   function Node_Id_To_Json (Id : Node_Id) return JSON_Value;
   function Source_Ptr_To_Json (P : Source_Ptr) return JSON_Value;
   function Symbol_To_Json (S : Symbols.Symbol) return JSON_Value;
   function Symbol_Set_To_Json (S : Symbol_Set) return JSON_Value;

   function Node_Id_To_Json (Id : Node_Id) return JSON_Value is
     (Create ("<Node_Id>")); --  TODO

   function Source_Ptr_To_Json (P : Source_Ptr) return JSON_Value is
      Res  : constant JSON_Value           := Create (Empty_Array);
      File : constant String               := SPARK_Util.File_Name (P);
      Line : constant Physical_Line_Number := Get_Physical_Line_Number (P);
   begin
      if P = No_Location then
         return Create;
      else
         Append (Res, Create (File));
         Append (Res, Create (Integer (Line)));
         return Res;
      end if;
   end Source_Ptr_To_Json;

   function Symbol_To_Json (S : Symbols.Symbol) return JSON_Value is
      P : constant GNATCOLL.Utils.Cst_String_Access := Symbols.Get (S);
   begin
      return Create (P.all);
   end Symbol_To_Json;

   function Symbol_Set_To_Json (S : Symbol_Set) return JSON_Value is
      Res : constant JSON_Value := Create (Empty_Array);
   begin
      for El of S loop
         Append (Res, Symbol_To_Json (El));
      end loop;
      return Res;
   end Symbol_Set_To_Json;

   -----------------------------------------
   --  Why nodes types with multiplicity  --
   -----------------------------------------

   function Why_Node_Id_To_Json (Id : Why_Node_Id) return JSON_Value;
   function Why_Node_OId_To_Json (Id : Why_Node_Id) return JSON_Value;
   function Why_Node_OList_To_Json (Id : Why_Node_List) return JSON_Value;
   function Why_Node_List_To_Json (Id : Why_Node_List) return JSON_Value;
   function Why_Node_Set_To_Json (S : Why_Node_Set) return JSON_Value;

   function Why_Node_Id_To_Json (Id : Why_Node_Id) return JSON_Value is
     (Why_Node_To_Json (Get_Node (Id)));

   function Why_Node_OId_To_Json (Id : Why_Node_Id) return JSON_Value is
     (if Id = Why_Empty then Create else Why_Node_Id_To_Json (Id));

   function Why_Node_OList_To_Json (Id : Why_Node_List) return JSON_Value is
     (if Is_Empty (Id) then Create else Why_Node_List_To_Json (Id));

   function Why_Node_List_To_Json (Id : Why_Node_List) return JSON_Value is
     (Why_Node_Lists_List_To_Json (Get_List (Id)));

   function Why_Node_Lists_List_To_Json
     (L : Why_Node_Lists.List) return JSON_Value
   is
      Res : constant JSON_Value := Create (Empty_Array);
   begin
      for El of L loop
         Append (Res, Why_Node_Id_To_Json (El));
      end loop;
      return Res;
   end Why_Node_Lists_List_To_Json;

   function Why_Node_Set_To_Json (S : Why_Node_Set) return JSON_Value is
     (Create ("<Why_Node_Set>")); --  TODO

   --  Why.Sinfo

   function EW_Domain_To_Json (Arg : EW_Domain)
      return JSON_Value;

   function EW_Domain_To_Json (Arg : EW_Domain)
   return JSON_Value
      is (Create (Integer (case Arg is
         when EW_Term => 1,
         when EW_Pterm => 2,
         when EW_Pred => 3,
         when EW_Prog => 4)));

   function EW_Type_To_Json (Arg : EW_Type)
      return JSON_Value;

   function EW_Type_To_Json (Arg : EW_Type)
   return JSON_Value
      is (Create (Integer (case Arg is
         when EW_Builtin => 0,
         when EW_Split => 1,
         when EW_Abstract => 2)));

   function EW_Literal_To_Json (Arg : EW_Literal)
      return JSON_Value;

   function EW_Literal_To_Json (Arg : EW_Literal)
   return JSON_Value
      is (Create (Integer (case Arg is
         when EW_True => 0,
         when EW_False => 1)));

   function EW_Theory_Type_To_Json (Arg : EW_Theory_Type)
      return JSON_Value;

   function EW_Theory_Type_To_Json (Arg : EW_Theory_Type)
   return JSON_Value
      is (Create (Integer (case Arg is
         when EW_Theory => 0,
         when EW_Module => 1)));

   function EW_Clone_Type_To_Json (Arg : EW_Clone_Type)
      return JSON_Value;

   function EW_Clone_Type_To_Json (Arg : EW_Clone_Type)
   return JSON_Value
      is (Create (Integer (case Arg is
         when EW_Import => 0,
         when EW_Export => 1,
         when EW_Clone_Default => 2)));

   function EW_Subst_Type_To_Json (Arg : EW_Subst_Type)
      return JSON_Value;

   function EW_Subst_Type_To_Json (Arg : EW_Subst_Type)
   return JSON_Value
      is (Create (Integer (case Arg is
         when EW_Type_Subst => 0,
         when EW_Function => 1,
         when EW_Predicate => 2,
         when EW_Namepace => 3,
         when EW_Lemma => 4,
         when EW_Goal => 5)));

   function EW_Connector_To_Json (Arg : EW_Connector)
      return JSON_Value;

   function EW_Connector_To_Json (Arg : EW_Connector)
   return JSON_Value
      is (Create (Integer (case Arg is
         when EW_Or_Else => 0,
         when EW_And_Then => 1,
         when EW_Imply => 2,
         when EW_Equivalent => 3,
         when EW_Or => 4,
         when EW_And => 5)));

   function EW_Assert_Kind_To_Json (Arg : EW_Assert_Kind)
      return JSON_Value;

   function EW_Assert_Kind_To_Json (Arg : EW_Assert_Kind)
   return JSON_Value
      is (Create (Integer (case Arg is
         when EW_Assert => 0,
         when EW_Check => 1,
         when EW_Assume => 2)));

   function EW_Axiom_Dep_Kind_To_Json (Arg : EW_Axiom_Dep_Kind)
      return JSON_Value;

   function EW_Axiom_Dep_Kind_To_Json (Arg : EW_Axiom_Dep_Kind)
   return JSON_Value
      is (Create (Integer (case Arg is
         when EW_Axdep_Func => 0,
         when EW_Axdep_Pred => 1)));

   function W_Unused_At_Start_Opaque_OId_To_Json
      (Arg : W_Unused_At_Start_Opaque_OId)
      return JSON_Value;

   function W_Unused_At_Start_Opaque_OId_To_Json
      (Arg : W_Unused_At_Start_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Unused_At_Start_Opaque_OList_To_Json
      (Arg : W_Unused_At_Start_Opaque_OList)
      return JSON_Value;

   function W_Unused_At_Start_Opaque_OList_To_Json
      (Arg : W_Unused_At_Start_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Unused_At_Start_Opaque_Id_To_Json
      (Arg : W_Unused_At_Start_Opaque_Id)
      return JSON_Value;

   function W_Unused_At_Start_Opaque_Id_To_Json
      (Arg : W_Unused_At_Start_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Unused_At_Start_Opaque_List_To_Json
      (Arg : W_Unused_At_Start_Opaque_List)
      return JSON_Value;

   function W_Unused_At_Start_Opaque_List_To_Json
      (Arg : W_Unused_At_Start_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Type_Opaque_OId_To_Json
      (Arg : W_Type_Opaque_OId)
      return JSON_Value;

   function W_Type_Opaque_OId_To_Json
      (Arg : W_Type_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Type_Opaque_OList_To_Json
      (Arg : W_Type_Opaque_OList)
      return JSON_Value;

   function W_Type_Opaque_OList_To_Json
      (Arg : W_Type_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Type_Opaque_Id_To_Json
      (Arg : W_Type_Opaque_Id)
      return JSON_Value;

   function W_Type_Opaque_Id_To_Json
      (Arg : W_Type_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Type_Opaque_List_To_Json
      (Arg : W_Type_Opaque_List)
      return JSON_Value;

   function W_Type_Opaque_List_To_Json
      (Arg : W_Type_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Name_Opaque_OId_To_Json
      (Arg : W_Name_Opaque_OId)
      return JSON_Value;

   function W_Name_Opaque_OId_To_Json
      (Arg : W_Name_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Name_Opaque_OList_To_Json
      (Arg : W_Name_Opaque_OList)
      return JSON_Value;

   function W_Name_Opaque_OList_To_Json
      (Arg : W_Name_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Name_Opaque_Id_To_Json
      (Arg : W_Name_Opaque_Id)
      return JSON_Value;

   function W_Name_Opaque_Id_To_Json
      (Arg : W_Name_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Name_Opaque_List_To_Json
      (Arg : W_Name_Opaque_List)
      return JSON_Value;

   function W_Name_Opaque_List_To_Json
      (Arg : W_Name_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Effects_Opaque_OId_To_Json
      (Arg : W_Effects_Opaque_OId)
      return JSON_Value;

   function W_Effects_Opaque_OId_To_Json
      (Arg : W_Effects_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Effects_Opaque_OList_To_Json
      (Arg : W_Effects_Opaque_OList)
      return JSON_Value;

   function W_Effects_Opaque_OList_To_Json
      (Arg : W_Effects_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Effects_Opaque_Id_To_Json
      (Arg : W_Effects_Opaque_Id)
      return JSON_Value;

   function W_Effects_Opaque_Id_To_Json
      (Arg : W_Effects_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Effects_Opaque_List_To_Json
      (Arg : W_Effects_Opaque_List)
      return JSON_Value;

   function W_Effects_Opaque_List_To_Json
      (Arg : W_Effects_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Raise_Effect_Opaque_OId_To_Json
      (Arg : W_Raise_Effect_Opaque_OId)
      return JSON_Value;

   function W_Raise_Effect_Opaque_OId_To_Json
      (Arg : W_Raise_Effect_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Raise_Effect_Opaque_OList_To_Json
      (Arg : W_Raise_Effect_Opaque_OList)
      return JSON_Value;

   function W_Raise_Effect_Opaque_OList_To_Json
      (Arg : W_Raise_Effect_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Raise_Effect_Opaque_Id_To_Json
      (Arg : W_Raise_Effect_Opaque_Id)
      return JSON_Value;

   function W_Raise_Effect_Opaque_Id_To_Json
      (Arg : W_Raise_Effect_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Raise_Effect_Opaque_List_To_Json
      (Arg : W_Raise_Effect_Opaque_List)
      return JSON_Value;

   function W_Raise_Effect_Opaque_List_To_Json
      (Arg : W_Raise_Effect_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Binder_Opaque_OId_To_Json
      (Arg : W_Binder_Opaque_OId)
      return JSON_Value;

   function W_Binder_Opaque_OId_To_Json
      (Arg : W_Binder_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Binder_Opaque_OList_To_Json
      (Arg : W_Binder_Opaque_OList)
      return JSON_Value;

   function W_Binder_Opaque_OList_To_Json
      (Arg : W_Binder_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Binder_Opaque_Id_To_Json
      (Arg : W_Binder_Opaque_Id)
      return JSON_Value;

   function W_Binder_Opaque_Id_To_Json
      (Arg : W_Binder_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Binder_Opaque_List_To_Json
      (Arg : W_Binder_Opaque_List)
      return JSON_Value;

   function W_Binder_Opaque_List_To_Json
      (Arg : W_Binder_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Transparent_Type_Definition_Opaque_OId_To_Json
      (Arg : W_Transparent_Type_Definition_Opaque_OId)
      return JSON_Value;

   function W_Transparent_Type_Definition_Opaque_OId_To_Json
      (Arg : W_Transparent_Type_Definition_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Transparent_Type_Definition_Opaque_OList_To_Json
      (Arg : W_Transparent_Type_Definition_Opaque_OList)
      return JSON_Value;

   function W_Transparent_Type_Definition_Opaque_OList_To_Json
      (Arg : W_Transparent_Type_Definition_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Transparent_Type_Definition_Opaque_Id_To_Json
      (Arg : W_Transparent_Type_Definition_Opaque_Id)
      return JSON_Value;

   function W_Transparent_Type_Definition_Opaque_Id_To_Json
      (Arg : W_Transparent_Type_Definition_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Transparent_Type_Definition_Opaque_List_To_Json
      (Arg : W_Transparent_Type_Definition_Opaque_List)
      return JSON_Value;

   function W_Transparent_Type_Definition_Opaque_List_To_Json
      (Arg : W_Transparent_Type_Definition_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Record_Binder_Opaque_OId_To_Json
      (Arg : W_Record_Binder_Opaque_OId)
      return JSON_Value;

   function W_Record_Binder_Opaque_OId_To_Json
      (Arg : W_Record_Binder_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Record_Binder_Opaque_OList_To_Json
      (Arg : W_Record_Binder_Opaque_OList)
      return JSON_Value;

   function W_Record_Binder_Opaque_OList_To_Json
      (Arg : W_Record_Binder_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Record_Binder_Opaque_Id_To_Json
      (Arg : W_Record_Binder_Opaque_Id)
      return JSON_Value;

   function W_Record_Binder_Opaque_Id_To_Json
      (Arg : W_Record_Binder_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Record_Binder_Opaque_List_To_Json
      (Arg : W_Record_Binder_Opaque_List)
      return JSON_Value;

   function W_Record_Binder_Opaque_List_To_Json
      (Arg : W_Record_Binder_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Record_Definition_Opaque_OId_To_Json
      (Arg : W_Record_Definition_Opaque_OId)
      return JSON_Value;

   function W_Record_Definition_Opaque_OId_To_Json
      (Arg : W_Record_Definition_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Record_Definition_Opaque_OList_To_Json
      (Arg : W_Record_Definition_Opaque_OList)
      return JSON_Value;

   function W_Record_Definition_Opaque_OList_To_Json
      (Arg : W_Record_Definition_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Record_Definition_Opaque_Id_To_Json
      (Arg : W_Record_Definition_Opaque_Id)
      return JSON_Value;

   function W_Record_Definition_Opaque_Id_To_Json
      (Arg : W_Record_Definition_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Record_Definition_Opaque_List_To_Json
      (Arg : W_Record_Definition_Opaque_List)
      return JSON_Value;

   function W_Record_Definition_Opaque_List_To_Json
      (Arg : W_Record_Definition_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Range_Type_Definition_Opaque_OId_To_Json
      (Arg : W_Range_Type_Definition_Opaque_OId)
      return JSON_Value;

   function W_Range_Type_Definition_Opaque_OId_To_Json
      (Arg : W_Range_Type_Definition_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Range_Type_Definition_Opaque_OList_To_Json
      (Arg : W_Range_Type_Definition_Opaque_OList)
      return JSON_Value;

   function W_Range_Type_Definition_Opaque_OList_To_Json
      (Arg : W_Range_Type_Definition_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Range_Type_Definition_Opaque_Id_To_Json
      (Arg : W_Range_Type_Definition_Opaque_Id)
      return JSON_Value;

   function W_Range_Type_Definition_Opaque_Id_To_Json
      (Arg : W_Range_Type_Definition_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Range_Type_Definition_Opaque_List_To_Json
      (Arg : W_Range_Type_Definition_Opaque_List)
      return JSON_Value;

   function W_Range_Type_Definition_Opaque_List_To_Json
      (Arg : W_Range_Type_Definition_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Triggers_Opaque_OId_To_Json
      (Arg : W_Triggers_Opaque_OId)
      return JSON_Value;

   function W_Triggers_Opaque_OId_To_Json
      (Arg : W_Triggers_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Triggers_Opaque_OList_To_Json
      (Arg : W_Triggers_Opaque_OList)
      return JSON_Value;

   function W_Triggers_Opaque_OList_To_Json
      (Arg : W_Triggers_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Triggers_Opaque_Id_To_Json
      (Arg : W_Triggers_Opaque_Id)
      return JSON_Value;

   function W_Triggers_Opaque_Id_To_Json
      (Arg : W_Triggers_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Triggers_Opaque_List_To_Json
      (Arg : W_Triggers_Opaque_List)
      return JSON_Value;

   function W_Triggers_Opaque_List_To_Json
      (Arg : W_Triggers_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Trigger_Opaque_OId_To_Json
      (Arg : W_Trigger_Opaque_OId)
      return JSON_Value;

   function W_Trigger_Opaque_OId_To_Json
      (Arg : W_Trigger_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Trigger_Opaque_OList_To_Json
      (Arg : W_Trigger_Opaque_OList)
      return JSON_Value;

   function W_Trigger_Opaque_OList_To_Json
      (Arg : W_Trigger_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Trigger_Opaque_Id_To_Json
      (Arg : W_Trigger_Opaque_Id)
      return JSON_Value;

   function W_Trigger_Opaque_Id_To_Json
      (Arg : W_Trigger_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Trigger_Opaque_List_To_Json
      (Arg : W_Trigger_Opaque_List)
      return JSON_Value;

   function W_Trigger_Opaque_List_To_Json
      (Arg : W_Trigger_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Axiom_Dep_Opaque_OId_To_Json
      (Arg : W_Axiom_Dep_Opaque_OId)
      return JSON_Value;

   function W_Axiom_Dep_Opaque_OId_To_Json
      (Arg : W_Axiom_Dep_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Axiom_Dep_Opaque_OList_To_Json
      (Arg : W_Axiom_Dep_Opaque_OList)
      return JSON_Value;

   function W_Axiom_Dep_Opaque_OList_To_Json
      (Arg : W_Axiom_Dep_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Axiom_Dep_Opaque_Id_To_Json
      (Arg : W_Axiom_Dep_Opaque_Id)
      return JSON_Value;

   function W_Axiom_Dep_Opaque_Id_To_Json
      (Arg : W_Axiom_Dep_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Axiom_Dep_Opaque_List_To_Json
      (Arg : W_Axiom_Dep_Opaque_List)
      return JSON_Value;

   function W_Axiom_Dep_Opaque_List_To_Json
      (Arg : W_Axiom_Dep_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Handler_Opaque_OId_To_Json
      (Arg : W_Handler_Opaque_OId)
      return JSON_Value;

   function W_Handler_Opaque_OId_To_Json
      (Arg : W_Handler_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Handler_Opaque_OList_To_Json
      (Arg : W_Handler_Opaque_OList)
      return JSON_Value;

   function W_Handler_Opaque_OList_To_Json
      (Arg : W_Handler_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Handler_Opaque_Id_To_Json
      (Arg : W_Handler_Opaque_Id)
      return JSON_Value;

   function W_Handler_Opaque_Id_To_Json
      (Arg : W_Handler_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Handler_Opaque_List_To_Json
      (Arg : W_Handler_Opaque_List)
      return JSON_Value;

   function W_Handler_Opaque_List_To_Json
      (Arg : W_Handler_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Field_Association_Opaque_OId_To_Json
      (Arg : W_Field_Association_Opaque_OId)
      return JSON_Value;

   function W_Field_Association_Opaque_OId_To_Json
      (Arg : W_Field_Association_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Field_Association_Opaque_OList_To_Json
      (Arg : W_Field_Association_Opaque_OList)
      return JSON_Value;

   function W_Field_Association_Opaque_OList_To_Json
      (Arg : W_Field_Association_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Field_Association_Opaque_Id_To_Json
      (Arg : W_Field_Association_Opaque_Id)
      return JSON_Value;

   function W_Field_Association_Opaque_Id_To_Json
      (Arg : W_Field_Association_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Field_Association_Opaque_List_To_Json
      (Arg : W_Field_Association_Opaque_List)
      return JSON_Value;

   function W_Field_Association_Opaque_List_To_Json
      (Arg : W_Field_Association_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Variant_Opaque_OId_To_Json
      (Arg : W_Variant_Opaque_OId)
      return JSON_Value;

   function W_Variant_Opaque_OId_To_Json
      (Arg : W_Variant_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Variant_Opaque_OList_To_Json
      (Arg : W_Variant_Opaque_OList)
      return JSON_Value;

   function W_Variant_Opaque_OList_To_Json
      (Arg : W_Variant_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Variant_Opaque_Id_To_Json
      (Arg : W_Variant_Opaque_Id)
      return JSON_Value;

   function W_Variant_Opaque_Id_To_Json
      (Arg : W_Variant_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Variant_Opaque_List_To_Json
      (Arg : W_Variant_Opaque_List)
      return JSON_Value;

   function W_Variant_Opaque_List_To_Json
      (Arg : W_Variant_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Variants_Opaque_OId_To_Json
      (Arg : W_Variants_Opaque_OId)
      return JSON_Value;

   function W_Variants_Opaque_OId_To_Json
      (Arg : W_Variants_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Variants_Opaque_OList_To_Json
      (Arg : W_Variants_Opaque_OList)
      return JSON_Value;

   function W_Variants_Opaque_OList_To_Json
      (Arg : W_Variants_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Variants_Opaque_Id_To_Json
      (Arg : W_Variants_Opaque_Id)
      return JSON_Value;

   function W_Variants_Opaque_Id_To_Json
      (Arg : W_Variants_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Variants_Opaque_List_To_Json
      (Arg : W_Variants_Opaque_List)
      return JSON_Value;

   function W_Variants_Opaque_List_To_Json
      (Arg : W_Variants_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Universal_Quantif_Opaque_OId_To_Json
      (Arg : W_Universal_Quantif_Opaque_OId)
      return JSON_Value;

   function W_Universal_Quantif_Opaque_OId_To_Json
      (Arg : W_Universal_Quantif_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Universal_Quantif_Opaque_OList_To_Json
      (Arg : W_Universal_Quantif_Opaque_OList)
      return JSON_Value;

   function W_Universal_Quantif_Opaque_OList_To_Json
      (Arg : W_Universal_Quantif_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Universal_Quantif_Opaque_Id_To_Json
      (Arg : W_Universal_Quantif_Opaque_Id)
      return JSON_Value;

   function W_Universal_Quantif_Opaque_Id_To_Json
      (Arg : W_Universal_Quantif_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Universal_Quantif_Opaque_List_To_Json
      (Arg : W_Universal_Quantif_Opaque_List)
      return JSON_Value;

   function W_Universal_Quantif_Opaque_List_To_Json
      (Arg : W_Universal_Quantif_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Existential_Quantif_Opaque_OId_To_Json
      (Arg : W_Existential_Quantif_Opaque_OId)
      return JSON_Value;

   function W_Existential_Quantif_Opaque_OId_To_Json
      (Arg : W_Existential_Quantif_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Existential_Quantif_Opaque_OList_To_Json
      (Arg : W_Existential_Quantif_Opaque_OList)
      return JSON_Value;

   function W_Existential_Quantif_Opaque_OList_To_Json
      (Arg : W_Existential_Quantif_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Existential_Quantif_Opaque_Id_To_Json
      (Arg : W_Existential_Quantif_Opaque_Id)
      return JSON_Value;

   function W_Existential_Quantif_Opaque_Id_To_Json
      (Arg : W_Existential_Quantif_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Existential_Quantif_Opaque_List_To_Json
      (Arg : W_Existential_Quantif_Opaque_List)
      return JSON_Value;

   function W_Existential_Quantif_Opaque_List_To_Json
      (Arg : W_Existential_Quantif_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Not_Opaque_OId_To_Json
      (Arg : W_Not_Opaque_OId)
      return JSON_Value;

   function W_Not_Opaque_OId_To_Json
      (Arg : W_Not_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Not_Opaque_OList_To_Json
      (Arg : W_Not_Opaque_OList)
      return JSON_Value;

   function W_Not_Opaque_OList_To_Json
      (Arg : W_Not_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Not_Opaque_Id_To_Json
      (Arg : W_Not_Opaque_Id)
      return JSON_Value;

   function W_Not_Opaque_Id_To_Json
      (Arg : W_Not_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Not_Opaque_List_To_Json
      (Arg : W_Not_Opaque_List)
      return JSON_Value;

   function W_Not_Opaque_List_To_Json
      (Arg : W_Not_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Connection_Opaque_OId_To_Json
      (Arg : W_Connection_Opaque_OId)
      return JSON_Value;

   function W_Connection_Opaque_OId_To_Json
      (Arg : W_Connection_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Connection_Opaque_OList_To_Json
      (Arg : W_Connection_Opaque_OList)
      return JSON_Value;

   function W_Connection_Opaque_OList_To_Json
      (Arg : W_Connection_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Connection_Opaque_Id_To_Json
      (Arg : W_Connection_Opaque_Id)
      return JSON_Value;

   function W_Connection_Opaque_Id_To_Json
      (Arg : W_Connection_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Connection_Opaque_List_To_Json
      (Arg : W_Connection_Opaque_List)
      return JSON_Value;

   function W_Connection_Opaque_List_To_Json
      (Arg : W_Connection_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Label_Opaque_OId_To_Json
      (Arg : W_Label_Opaque_OId)
      return JSON_Value;

   function W_Label_Opaque_OId_To_Json
      (Arg : W_Label_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Label_Opaque_OList_To_Json
      (Arg : W_Label_Opaque_OList)
      return JSON_Value;

   function W_Label_Opaque_OList_To_Json
      (Arg : W_Label_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Label_Opaque_Id_To_Json
      (Arg : W_Label_Opaque_Id)
      return JSON_Value;

   function W_Label_Opaque_Id_To_Json
      (Arg : W_Label_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Label_Opaque_List_To_Json
      (Arg : W_Label_Opaque_List)
      return JSON_Value;

   function W_Label_Opaque_List_To_Json
      (Arg : W_Label_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Loc_Label_Opaque_OId_To_Json
      (Arg : W_Loc_Label_Opaque_OId)
      return JSON_Value;

   function W_Loc_Label_Opaque_OId_To_Json
      (Arg : W_Loc_Label_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Loc_Label_Opaque_OList_To_Json
      (Arg : W_Loc_Label_Opaque_OList)
      return JSON_Value;

   function W_Loc_Label_Opaque_OList_To_Json
      (Arg : W_Loc_Label_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Loc_Label_Opaque_Id_To_Json
      (Arg : W_Loc_Label_Opaque_Id)
      return JSON_Value;

   function W_Loc_Label_Opaque_Id_To_Json
      (Arg : W_Loc_Label_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Loc_Label_Opaque_List_To_Json
      (Arg : W_Loc_Label_Opaque_List)
      return JSON_Value;

   function W_Loc_Label_Opaque_List_To_Json
      (Arg : W_Loc_Label_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Identifier_Opaque_OId_To_Json
      (Arg : W_Identifier_Opaque_OId)
      return JSON_Value;

   function W_Identifier_Opaque_OId_To_Json
      (Arg : W_Identifier_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Identifier_Opaque_OList_To_Json
      (Arg : W_Identifier_Opaque_OList)
      return JSON_Value;

   function W_Identifier_Opaque_OList_To_Json
      (Arg : W_Identifier_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Identifier_Opaque_Id_To_Json
      (Arg : W_Identifier_Opaque_Id)
      return JSON_Value;

   function W_Identifier_Opaque_Id_To_Json
      (Arg : W_Identifier_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Identifier_Opaque_List_To_Json
      (Arg : W_Identifier_Opaque_List)
      return JSON_Value;

   function W_Identifier_Opaque_List_To_Json
      (Arg : W_Identifier_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Tagged_Opaque_OId_To_Json
      (Arg : W_Tagged_Opaque_OId)
      return JSON_Value;

   function W_Tagged_Opaque_OId_To_Json
      (Arg : W_Tagged_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Tagged_Opaque_OList_To_Json
      (Arg : W_Tagged_Opaque_OList)
      return JSON_Value;

   function W_Tagged_Opaque_OList_To_Json
      (Arg : W_Tagged_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Tagged_Opaque_Id_To_Json
      (Arg : W_Tagged_Opaque_Id)
      return JSON_Value;

   function W_Tagged_Opaque_Id_To_Json
      (Arg : W_Tagged_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Tagged_Opaque_List_To_Json
      (Arg : W_Tagged_Opaque_List)
      return JSON_Value;

   function W_Tagged_Opaque_List_To_Json
      (Arg : W_Tagged_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Call_Opaque_OId_To_Json
      (Arg : W_Call_Opaque_OId)
      return JSON_Value;

   function W_Call_Opaque_OId_To_Json
      (Arg : W_Call_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Call_Opaque_OList_To_Json
      (Arg : W_Call_Opaque_OList)
      return JSON_Value;

   function W_Call_Opaque_OList_To_Json
      (Arg : W_Call_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Call_Opaque_Id_To_Json
      (Arg : W_Call_Opaque_Id)
      return JSON_Value;

   function W_Call_Opaque_Id_To_Json
      (Arg : W_Call_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Call_Opaque_List_To_Json
      (Arg : W_Call_Opaque_List)
      return JSON_Value;

   function W_Call_Opaque_List_To_Json
      (Arg : W_Call_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Literal_Opaque_OId_To_Json
      (Arg : W_Literal_Opaque_OId)
      return JSON_Value;

   function W_Literal_Opaque_OId_To_Json
      (Arg : W_Literal_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Literal_Opaque_OList_To_Json
      (Arg : W_Literal_Opaque_OList)
      return JSON_Value;

   function W_Literal_Opaque_OList_To_Json
      (Arg : W_Literal_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Literal_Opaque_Id_To_Json
      (Arg : W_Literal_Opaque_Id)
      return JSON_Value;

   function W_Literal_Opaque_Id_To_Json
      (Arg : W_Literal_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Literal_Opaque_List_To_Json
      (Arg : W_Literal_Opaque_List)
      return JSON_Value;

   function W_Literal_Opaque_List_To_Json
      (Arg : W_Literal_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Binding_Opaque_OId_To_Json
      (Arg : W_Binding_Opaque_OId)
      return JSON_Value;

   function W_Binding_Opaque_OId_To_Json
      (Arg : W_Binding_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Binding_Opaque_OList_To_Json
      (Arg : W_Binding_Opaque_OList)
      return JSON_Value;

   function W_Binding_Opaque_OList_To_Json
      (Arg : W_Binding_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Binding_Opaque_Id_To_Json
      (Arg : W_Binding_Opaque_Id)
      return JSON_Value;

   function W_Binding_Opaque_Id_To_Json
      (Arg : W_Binding_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Binding_Opaque_List_To_Json
      (Arg : W_Binding_Opaque_List)
      return JSON_Value;

   function W_Binding_Opaque_List_To_Json
      (Arg : W_Binding_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Elsif_Opaque_OId_To_Json
      (Arg : W_Elsif_Opaque_OId)
      return JSON_Value;

   function W_Elsif_Opaque_OId_To_Json
      (Arg : W_Elsif_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Elsif_Opaque_OList_To_Json
      (Arg : W_Elsif_Opaque_OList)
      return JSON_Value;

   function W_Elsif_Opaque_OList_To_Json
      (Arg : W_Elsif_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Elsif_Opaque_Id_To_Json
      (Arg : W_Elsif_Opaque_Id)
      return JSON_Value;

   function W_Elsif_Opaque_Id_To_Json
      (Arg : W_Elsif_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Elsif_Opaque_List_To_Json
      (Arg : W_Elsif_Opaque_List)
      return JSON_Value;

   function W_Elsif_Opaque_List_To_Json
      (Arg : W_Elsif_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Epsilon_Opaque_OId_To_Json
      (Arg : W_Epsilon_Opaque_OId)
      return JSON_Value;

   function W_Epsilon_Opaque_OId_To_Json
      (Arg : W_Epsilon_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Epsilon_Opaque_OList_To_Json
      (Arg : W_Epsilon_Opaque_OList)
      return JSON_Value;

   function W_Epsilon_Opaque_OList_To_Json
      (Arg : W_Epsilon_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Epsilon_Opaque_Id_To_Json
      (Arg : W_Epsilon_Opaque_Id)
      return JSON_Value;

   function W_Epsilon_Opaque_Id_To_Json
      (Arg : W_Epsilon_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Epsilon_Opaque_List_To_Json
      (Arg : W_Epsilon_Opaque_List)
      return JSON_Value;

   function W_Epsilon_Opaque_List_To_Json
      (Arg : W_Epsilon_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Conditional_Opaque_OId_To_Json
      (Arg : W_Conditional_Opaque_OId)
      return JSON_Value;

   function W_Conditional_Opaque_OId_To_Json
      (Arg : W_Conditional_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Conditional_Opaque_OList_To_Json
      (Arg : W_Conditional_Opaque_OList)
      return JSON_Value;

   function W_Conditional_Opaque_OList_To_Json
      (Arg : W_Conditional_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Conditional_Opaque_Id_To_Json
      (Arg : W_Conditional_Opaque_Id)
      return JSON_Value;

   function W_Conditional_Opaque_Id_To_Json
      (Arg : W_Conditional_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Conditional_Opaque_List_To_Json
      (Arg : W_Conditional_Opaque_List)
      return JSON_Value;

   function W_Conditional_Opaque_List_To_Json
      (Arg : W_Conditional_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Integer_Constant_Opaque_OId_To_Json
      (Arg : W_Integer_Constant_Opaque_OId)
      return JSON_Value;

   function W_Integer_Constant_Opaque_OId_To_Json
      (Arg : W_Integer_Constant_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Integer_Constant_Opaque_OList_To_Json
      (Arg : W_Integer_Constant_Opaque_OList)
      return JSON_Value;

   function W_Integer_Constant_Opaque_OList_To_Json
      (Arg : W_Integer_Constant_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Integer_Constant_Opaque_Id_To_Json
      (Arg : W_Integer_Constant_Opaque_Id)
      return JSON_Value;

   function W_Integer_Constant_Opaque_Id_To_Json
      (Arg : W_Integer_Constant_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Integer_Constant_Opaque_List_To_Json
      (Arg : W_Integer_Constant_Opaque_List)
      return JSON_Value;

   function W_Integer_Constant_Opaque_List_To_Json
      (Arg : W_Integer_Constant_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Range_Constant_Opaque_OId_To_Json
      (Arg : W_Range_Constant_Opaque_OId)
      return JSON_Value;

   function W_Range_Constant_Opaque_OId_To_Json
      (Arg : W_Range_Constant_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Range_Constant_Opaque_OList_To_Json
      (Arg : W_Range_Constant_Opaque_OList)
      return JSON_Value;

   function W_Range_Constant_Opaque_OList_To_Json
      (Arg : W_Range_Constant_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Range_Constant_Opaque_Id_To_Json
      (Arg : W_Range_Constant_Opaque_Id)
      return JSON_Value;

   function W_Range_Constant_Opaque_Id_To_Json
      (Arg : W_Range_Constant_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Range_Constant_Opaque_List_To_Json
      (Arg : W_Range_Constant_Opaque_List)
      return JSON_Value;

   function W_Range_Constant_Opaque_List_To_Json
      (Arg : W_Range_Constant_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Modular_Constant_Opaque_OId_To_Json
      (Arg : W_Modular_Constant_Opaque_OId)
      return JSON_Value;

   function W_Modular_Constant_Opaque_OId_To_Json
      (Arg : W_Modular_Constant_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Modular_Constant_Opaque_OList_To_Json
      (Arg : W_Modular_Constant_Opaque_OList)
      return JSON_Value;

   function W_Modular_Constant_Opaque_OList_To_Json
      (Arg : W_Modular_Constant_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Modular_Constant_Opaque_Id_To_Json
      (Arg : W_Modular_Constant_Opaque_Id)
      return JSON_Value;

   function W_Modular_Constant_Opaque_Id_To_Json
      (Arg : W_Modular_Constant_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Modular_Constant_Opaque_List_To_Json
      (Arg : W_Modular_Constant_Opaque_List)
      return JSON_Value;

   function W_Modular_Constant_Opaque_List_To_Json
      (Arg : W_Modular_Constant_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Fixed_Constant_Opaque_OId_To_Json
      (Arg : W_Fixed_Constant_Opaque_OId)
      return JSON_Value;

   function W_Fixed_Constant_Opaque_OId_To_Json
      (Arg : W_Fixed_Constant_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Fixed_Constant_Opaque_OList_To_Json
      (Arg : W_Fixed_Constant_Opaque_OList)
      return JSON_Value;

   function W_Fixed_Constant_Opaque_OList_To_Json
      (Arg : W_Fixed_Constant_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Fixed_Constant_Opaque_Id_To_Json
      (Arg : W_Fixed_Constant_Opaque_Id)
      return JSON_Value;

   function W_Fixed_Constant_Opaque_Id_To_Json
      (Arg : W_Fixed_Constant_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Fixed_Constant_Opaque_List_To_Json
      (Arg : W_Fixed_Constant_Opaque_List)
      return JSON_Value;

   function W_Fixed_Constant_Opaque_List_To_Json
      (Arg : W_Fixed_Constant_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Real_Constant_Opaque_OId_To_Json
      (Arg : W_Real_Constant_Opaque_OId)
      return JSON_Value;

   function W_Real_Constant_Opaque_OId_To_Json
      (Arg : W_Real_Constant_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Real_Constant_Opaque_OList_To_Json
      (Arg : W_Real_Constant_Opaque_OList)
      return JSON_Value;

   function W_Real_Constant_Opaque_OList_To_Json
      (Arg : W_Real_Constant_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Real_Constant_Opaque_Id_To_Json
      (Arg : W_Real_Constant_Opaque_Id)
      return JSON_Value;

   function W_Real_Constant_Opaque_Id_To_Json
      (Arg : W_Real_Constant_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Real_Constant_Opaque_List_To_Json
      (Arg : W_Real_Constant_Opaque_List)
      return JSON_Value;

   function W_Real_Constant_Opaque_List_To_Json
      (Arg : W_Real_Constant_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Float_Constant_Opaque_OId_To_Json
      (Arg : W_Float_Constant_Opaque_OId)
      return JSON_Value;

   function W_Float_Constant_Opaque_OId_To_Json
      (Arg : W_Float_Constant_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Float_Constant_Opaque_OList_To_Json
      (Arg : W_Float_Constant_Opaque_OList)
      return JSON_Value;

   function W_Float_Constant_Opaque_OList_To_Json
      (Arg : W_Float_Constant_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Float_Constant_Opaque_Id_To_Json
      (Arg : W_Float_Constant_Opaque_Id)
      return JSON_Value;

   function W_Float_Constant_Opaque_Id_To_Json
      (Arg : W_Float_Constant_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Float_Constant_Opaque_List_To_Json
      (Arg : W_Float_Constant_Opaque_List)
      return JSON_Value;

   function W_Float_Constant_Opaque_List_To_Json
      (Arg : W_Float_Constant_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Comment_Opaque_OId_To_Json
      (Arg : W_Comment_Opaque_OId)
      return JSON_Value;

   function W_Comment_Opaque_OId_To_Json
      (Arg : W_Comment_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Comment_Opaque_OList_To_Json
      (Arg : W_Comment_Opaque_OList)
      return JSON_Value;

   function W_Comment_Opaque_OList_To_Json
      (Arg : W_Comment_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Comment_Opaque_Id_To_Json
      (Arg : W_Comment_Opaque_Id)
      return JSON_Value;

   function W_Comment_Opaque_Id_To_Json
      (Arg : W_Comment_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Comment_Opaque_List_To_Json
      (Arg : W_Comment_Opaque_List)
      return JSON_Value;

   function W_Comment_Opaque_List_To_Json
      (Arg : W_Comment_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Deref_Opaque_OId_To_Json
      (Arg : W_Deref_Opaque_OId)
      return JSON_Value;

   function W_Deref_Opaque_OId_To_Json
      (Arg : W_Deref_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Deref_Opaque_OList_To_Json
      (Arg : W_Deref_Opaque_OList)
      return JSON_Value;

   function W_Deref_Opaque_OList_To_Json
      (Arg : W_Deref_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Deref_Opaque_Id_To_Json
      (Arg : W_Deref_Opaque_Id)
      return JSON_Value;

   function W_Deref_Opaque_Id_To_Json
      (Arg : W_Deref_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Deref_Opaque_List_To_Json
      (Arg : W_Deref_Opaque_List)
      return JSON_Value;

   function W_Deref_Opaque_List_To_Json
      (Arg : W_Deref_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Record_Access_Opaque_OId_To_Json
      (Arg : W_Record_Access_Opaque_OId)
      return JSON_Value;

   function W_Record_Access_Opaque_OId_To_Json
      (Arg : W_Record_Access_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Record_Access_Opaque_OList_To_Json
      (Arg : W_Record_Access_Opaque_OList)
      return JSON_Value;

   function W_Record_Access_Opaque_OList_To_Json
      (Arg : W_Record_Access_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Record_Access_Opaque_Id_To_Json
      (Arg : W_Record_Access_Opaque_Id)
      return JSON_Value;

   function W_Record_Access_Opaque_Id_To_Json
      (Arg : W_Record_Access_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Record_Access_Opaque_List_To_Json
      (Arg : W_Record_Access_Opaque_List)
      return JSON_Value;

   function W_Record_Access_Opaque_List_To_Json
      (Arg : W_Record_Access_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Record_Update_Opaque_OId_To_Json
      (Arg : W_Record_Update_Opaque_OId)
      return JSON_Value;

   function W_Record_Update_Opaque_OId_To_Json
      (Arg : W_Record_Update_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Record_Update_Opaque_OList_To_Json
      (Arg : W_Record_Update_Opaque_OList)
      return JSON_Value;

   function W_Record_Update_Opaque_OList_To_Json
      (Arg : W_Record_Update_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Record_Update_Opaque_Id_To_Json
      (Arg : W_Record_Update_Opaque_Id)
      return JSON_Value;

   function W_Record_Update_Opaque_Id_To_Json
      (Arg : W_Record_Update_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Record_Update_Opaque_List_To_Json
      (Arg : W_Record_Update_Opaque_List)
      return JSON_Value;

   function W_Record_Update_Opaque_List_To_Json
      (Arg : W_Record_Update_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Record_Aggregate_Opaque_OId_To_Json
      (Arg : W_Record_Aggregate_Opaque_OId)
      return JSON_Value;

   function W_Record_Aggregate_Opaque_OId_To_Json
      (Arg : W_Record_Aggregate_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Record_Aggregate_Opaque_OList_To_Json
      (Arg : W_Record_Aggregate_Opaque_OList)
      return JSON_Value;

   function W_Record_Aggregate_Opaque_OList_To_Json
      (Arg : W_Record_Aggregate_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Record_Aggregate_Opaque_Id_To_Json
      (Arg : W_Record_Aggregate_Opaque_Id)
      return JSON_Value;

   function W_Record_Aggregate_Opaque_Id_To_Json
      (Arg : W_Record_Aggregate_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Record_Aggregate_Opaque_List_To_Json
      (Arg : W_Record_Aggregate_Opaque_List)
      return JSON_Value;

   function W_Record_Aggregate_Opaque_List_To_Json
      (Arg : W_Record_Aggregate_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Any_Expr_Opaque_OId_To_Json
      (Arg : W_Any_Expr_Opaque_OId)
      return JSON_Value;

   function W_Any_Expr_Opaque_OId_To_Json
      (Arg : W_Any_Expr_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Any_Expr_Opaque_OList_To_Json
      (Arg : W_Any_Expr_Opaque_OList)
      return JSON_Value;

   function W_Any_Expr_Opaque_OList_To_Json
      (Arg : W_Any_Expr_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Any_Expr_Opaque_Id_To_Json
      (Arg : W_Any_Expr_Opaque_Id)
      return JSON_Value;

   function W_Any_Expr_Opaque_Id_To_Json
      (Arg : W_Any_Expr_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Any_Expr_Opaque_List_To_Json
      (Arg : W_Any_Expr_Opaque_List)
      return JSON_Value;

   function W_Any_Expr_Opaque_List_To_Json
      (Arg : W_Any_Expr_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Assignment_Opaque_OId_To_Json
      (Arg : W_Assignment_Opaque_OId)
      return JSON_Value;

   function W_Assignment_Opaque_OId_To_Json
      (Arg : W_Assignment_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Assignment_Opaque_OList_To_Json
      (Arg : W_Assignment_Opaque_OList)
      return JSON_Value;

   function W_Assignment_Opaque_OList_To_Json
      (Arg : W_Assignment_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Assignment_Opaque_Id_To_Json
      (Arg : W_Assignment_Opaque_Id)
      return JSON_Value;

   function W_Assignment_Opaque_Id_To_Json
      (Arg : W_Assignment_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Assignment_Opaque_List_To_Json
      (Arg : W_Assignment_Opaque_List)
      return JSON_Value;

   function W_Assignment_Opaque_List_To_Json
      (Arg : W_Assignment_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Binding_Ref_Opaque_OId_To_Json
      (Arg : W_Binding_Ref_Opaque_OId)
      return JSON_Value;

   function W_Binding_Ref_Opaque_OId_To_Json
      (Arg : W_Binding_Ref_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Binding_Ref_Opaque_OList_To_Json
      (Arg : W_Binding_Ref_Opaque_OList)
      return JSON_Value;

   function W_Binding_Ref_Opaque_OList_To_Json
      (Arg : W_Binding_Ref_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Binding_Ref_Opaque_Id_To_Json
      (Arg : W_Binding_Ref_Opaque_Id)
      return JSON_Value;

   function W_Binding_Ref_Opaque_Id_To_Json
      (Arg : W_Binding_Ref_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Binding_Ref_Opaque_List_To_Json
      (Arg : W_Binding_Ref_Opaque_List)
      return JSON_Value;

   function W_Binding_Ref_Opaque_List_To_Json
      (Arg : W_Binding_Ref_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Loop_Opaque_OId_To_Json
      (Arg : W_Loop_Opaque_OId)
      return JSON_Value;

   function W_Loop_Opaque_OId_To_Json
      (Arg : W_Loop_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Loop_Opaque_OList_To_Json
      (Arg : W_Loop_Opaque_OList)
      return JSON_Value;

   function W_Loop_Opaque_OList_To_Json
      (Arg : W_Loop_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Loop_Opaque_Id_To_Json
      (Arg : W_Loop_Opaque_Id)
      return JSON_Value;

   function W_Loop_Opaque_Id_To_Json
      (Arg : W_Loop_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Loop_Opaque_List_To_Json
      (Arg : W_Loop_Opaque_List)
      return JSON_Value;

   function W_Loop_Opaque_List_To_Json
      (Arg : W_Loop_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Statement_Sequence_Opaque_OId_To_Json
      (Arg : W_Statement_Sequence_Opaque_OId)
      return JSON_Value;

   function W_Statement_Sequence_Opaque_OId_To_Json
      (Arg : W_Statement_Sequence_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Statement_Sequence_Opaque_OList_To_Json
      (Arg : W_Statement_Sequence_Opaque_OList)
      return JSON_Value;

   function W_Statement_Sequence_Opaque_OList_To_Json
      (Arg : W_Statement_Sequence_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Statement_Sequence_Opaque_Id_To_Json
      (Arg : W_Statement_Sequence_Opaque_Id)
      return JSON_Value;

   function W_Statement_Sequence_Opaque_Id_To_Json
      (Arg : W_Statement_Sequence_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Statement_Sequence_Opaque_List_To_Json
      (Arg : W_Statement_Sequence_Opaque_List)
      return JSON_Value;

   function W_Statement_Sequence_Opaque_List_To_Json
      (Arg : W_Statement_Sequence_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Abstract_Expr_Opaque_OId_To_Json
      (Arg : W_Abstract_Expr_Opaque_OId)
      return JSON_Value;

   function W_Abstract_Expr_Opaque_OId_To_Json
      (Arg : W_Abstract_Expr_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Abstract_Expr_Opaque_OList_To_Json
      (Arg : W_Abstract_Expr_Opaque_OList)
      return JSON_Value;

   function W_Abstract_Expr_Opaque_OList_To_Json
      (Arg : W_Abstract_Expr_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Abstract_Expr_Opaque_Id_To_Json
      (Arg : W_Abstract_Expr_Opaque_Id)
      return JSON_Value;

   function W_Abstract_Expr_Opaque_Id_To_Json
      (Arg : W_Abstract_Expr_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Abstract_Expr_Opaque_List_To_Json
      (Arg : W_Abstract_Expr_Opaque_List)
      return JSON_Value;

   function W_Abstract_Expr_Opaque_List_To_Json
      (Arg : W_Abstract_Expr_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Assert_Opaque_OId_To_Json
      (Arg : W_Assert_Opaque_OId)
      return JSON_Value;

   function W_Assert_Opaque_OId_To_Json
      (Arg : W_Assert_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Assert_Opaque_OList_To_Json
      (Arg : W_Assert_Opaque_OList)
      return JSON_Value;

   function W_Assert_Opaque_OList_To_Json
      (Arg : W_Assert_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Assert_Opaque_Id_To_Json
      (Arg : W_Assert_Opaque_Id)
      return JSON_Value;

   function W_Assert_Opaque_Id_To_Json
      (Arg : W_Assert_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Assert_Opaque_List_To_Json
      (Arg : W_Assert_Opaque_List)
      return JSON_Value;

   function W_Assert_Opaque_List_To_Json
      (Arg : W_Assert_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Raise_Opaque_OId_To_Json
      (Arg : W_Raise_Opaque_OId)
      return JSON_Value;

   function W_Raise_Opaque_OId_To_Json
      (Arg : W_Raise_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Raise_Opaque_OList_To_Json
      (Arg : W_Raise_Opaque_OList)
      return JSON_Value;

   function W_Raise_Opaque_OList_To_Json
      (Arg : W_Raise_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Raise_Opaque_Id_To_Json
      (Arg : W_Raise_Opaque_Id)
      return JSON_Value;

   function W_Raise_Opaque_Id_To_Json
      (Arg : W_Raise_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Raise_Opaque_List_To_Json
      (Arg : W_Raise_Opaque_List)
      return JSON_Value;

   function W_Raise_Opaque_List_To_Json
      (Arg : W_Raise_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Try_Block_Opaque_OId_To_Json
      (Arg : W_Try_Block_Opaque_OId)
      return JSON_Value;

   function W_Try_Block_Opaque_OId_To_Json
      (Arg : W_Try_Block_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Try_Block_Opaque_OList_To_Json
      (Arg : W_Try_Block_Opaque_OList)
      return JSON_Value;

   function W_Try_Block_Opaque_OList_To_Json
      (Arg : W_Try_Block_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Try_Block_Opaque_Id_To_Json
      (Arg : W_Try_Block_Opaque_Id)
      return JSON_Value;

   function W_Try_Block_Opaque_Id_To_Json
      (Arg : W_Try_Block_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Try_Block_Opaque_List_To_Json
      (Arg : W_Try_Block_Opaque_List)
      return JSON_Value;

   function W_Try_Block_Opaque_List_To_Json
      (Arg : W_Try_Block_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Function_Decl_Opaque_OId_To_Json
      (Arg : W_Function_Decl_Opaque_OId)
      return JSON_Value;

   function W_Function_Decl_Opaque_OId_To_Json
      (Arg : W_Function_Decl_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Function_Decl_Opaque_OList_To_Json
      (Arg : W_Function_Decl_Opaque_OList)
      return JSON_Value;

   function W_Function_Decl_Opaque_OList_To_Json
      (Arg : W_Function_Decl_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Function_Decl_Opaque_Id_To_Json
      (Arg : W_Function_Decl_Opaque_Id)
      return JSON_Value;

   function W_Function_Decl_Opaque_Id_To_Json
      (Arg : W_Function_Decl_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Function_Decl_Opaque_List_To_Json
      (Arg : W_Function_Decl_Opaque_List)
      return JSON_Value;

   function W_Function_Decl_Opaque_List_To_Json
      (Arg : W_Function_Decl_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Axiom_Opaque_OId_To_Json
      (Arg : W_Axiom_Opaque_OId)
      return JSON_Value;

   function W_Axiom_Opaque_OId_To_Json
      (Arg : W_Axiom_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Axiom_Opaque_OList_To_Json
      (Arg : W_Axiom_Opaque_OList)
      return JSON_Value;

   function W_Axiom_Opaque_OList_To_Json
      (Arg : W_Axiom_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Axiom_Opaque_Id_To_Json
      (Arg : W_Axiom_Opaque_Id)
      return JSON_Value;

   function W_Axiom_Opaque_Id_To_Json
      (Arg : W_Axiom_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Axiom_Opaque_List_To_Json
      (Arg : W_Axiom_Opaque_List)
      return JSON_Value;

   function W_Axiom_Opaque_List_To_Json
      (Arg : W_Axiom_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Goal_Opaque_OId_To_Json
      (Arg : W_Goal_Opaque_OId)
      return JSON_Value;

   function W_Goal_Opaque_OId_To_Json
      (Arg : W_Goal_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Goal_Opaque_OList_To_Json
      (Arg : W_Goal_Opaque_OList)
      return JSON_Value;

   function W_Goal_Opaque_OList_To_Json
      (Arg : W_Goal_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Goal_Opaque_Id_To_Json
      (Arg : W_Goal_Opaque_Id)
      return JSON_Value;

   function W_Goal_Opaque_Id_To_Json
      (Arg : W_Goal_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Goal_Opaque_List_To_Json
      (Arg : W_Goal_Opaque_List)
      return JSON_Value;

   function W_Goal_Opaque_List_To_Json
      (Arg : W_Goal_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Type_Decl_Opaque_OId_To_Json
      (Arg : W_Type_Decl_Opaque_OId)
      return JSON_Value;

   function W_Type_Decl_Opaque_OId_To_Json
      (Arg : W_Type_Decl_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Type_Decl_Opaque_OList_To_Json
      (Arg : W_Type_Decl_Opaque_OList)
      return JSON_Value;

   function W_Type_Decl_Opaque_OList_To_Json
      (Arg : W_Type_Decl_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Type_Decl_Opaque_Id_To_Json
      (Arg : W_Type_Decl_Opaque_Id)
      return JSON_Value;

   function W_Type_Decl_Opaque_Id_To_Json
      (Arg : W_Type_Decl_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Type_Decl_Opaque_List_To_Json
      (Arg : W_Type_Decl_Opaque_List)
      return JSON_Value;

   function W_Type_Decl_Opaque_List_To_Json
      (Arg : W_Type_Decl_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Global_Ref_Declaration_Opaque_OId_To_Json
      (Arg : W_Global_Ref_Declaration_Opaque_OId)
      return JSON_Value;

   function W_Global_Ref_Declaration_Opaque_OId_To_Json
      (Arg : W_Global_Ref_Declaration_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Global_Ref_Declaration_Opaque_OList_To_Json
      (Arg : W_Global_Ref_Declaration_Opaque_OList)
      return JSON_Value;

   function W_Global_Ref_Declaration_Opaque_OList_To_Json
      (Arg : W_Global_Ref_Declaration_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Global_Ref_Declaration_Opaque_Id_To_Json
      (Arg : W_Global_Ref_Declaration_Opaque_Id)
      return JSON_Value;

   function W_Global_Ref_Declaration_Opaque_Id_To_Json
      (Arg : W_Global_Ref_Declaration_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Global_Ref_Declaration_Opaque_List_To_Json
      (Arg : W_Global_Ref_Declaration_Opaque_List)
      return JSON_Value;

   function W_Global_Ref_Declaration_Opaque_List_To_Json
      (Arg : W_Global_Ref_Declaration_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Namespace_Declaration_Opaque_OId_To_Json
      (Arg : W_Namespace_Declaration_Opaque_OId)
      return JSON_Value;

   function W_Namespace_Declaration_Opaque_OId_To_Json
      (Arg : W_Namespace_Declaration_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Namespace_Declaration_Opaque_OList_To_Json
      (Arg : W_Namespace_Declaration_Opaque_OList)
      return JSON_Value;

   function W_Namespace_Declaration_Opaque_OList_To_Json
      (Arg : W_Namespace_Declaration_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Namespace_Declaration_Opaque_Id_To_Json
      (Arg : W_Namespace_Declaration_Opaque_Id)
      return JSON_Value;

   function W_Namespace_Declaration_Opaque_Id_To_Json
      (Arg : W_Namespace_Declaration_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Namespace_Declaration_Opaque_List_To_Json
      (Arg : W_Namespace_Declaration_Opaque_List)
      return JSON_Value;

   function W_Namespace_Declaration_Opaque_List_To_Json
      (Arg : W_Namespace_Declaration_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Exception_Declaration_Opaque_OId_To_Json
      (Arg : W_Exception_Declaration_Opaque_OId)
      return JSON_Value;

   function W_Exception_Declaration_Opaque_OId_To_Json
      (Arg : W_Exception_Declaration_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Exception_Declaration_Opaque_OList_To_Json
      (Arg : W_Exception_Declaration_Opaque_OList)
      return JSON_Value;

   function W_Exception_Declaration_Opaque_OList_To_Json
      (Arg : W_Exception_Declaration_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Exception_Declaration_Opaque_Id_To_Json
      (Arg : W_Exception_Declaration_Opaque_Id)
      return JSON_Value;

   function W_Exception_Declaration_Opaque_Id_To_Json
      (Arg : W_Exception_Declaration_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Exception_Declaration_Opaque_List_To_Json
      (Arg : W_Exception_Declaration_Opaque_List)
      return JSON_Value;

   function W_Exception_Declaration_Opaque_List_To_Json
      (Arg : W_Exception_Declaration_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Meta_Declaration_Opaque_OId_To_Json
      (Arg : W_Meta_Declaration_Opaque_OId)
      return JSON_Value;

   function W_Meta_Declaration_Opaque_OId_To_Json
      (Arg : W_Meta_Declaration_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Meta_Declaration_Opaque_OList_To_Json
      (Arg : W_Meta_Declaration_Opaque_OList)
      return JSON_Value;

   function W_Meta_Declaration_Opaque_OList_To_Json
      (Arg : W_Meta_Declaration_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Meta_Declaration_Opaque_Id_To_Json
      (Arg : W_Meta_Declaration_Opaque_Id)
      return JSON_Value;

   function W_Meta_Declaration_Opaque_Id_To_Json
      (Arg : W_Meta_Declaration_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Meta_Declaration_Opaque_List_To_Json
      (Arg : W_Meta_Declaration_Opaque_List)
      return JSON_Value;

   function W_Meta_Declaration_Opaque_List_To_Json
      (Arg : W_Meta_Declaration_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Clone_Declaration_Opaque_OId_To_Json
      (Arg : W_Clone_Declaration_Opaque_OId)
      return JSON_Value;

   function W_Clone_Declaration_Opaque_OId_To_Json
      (Arg : W_Clone_Declaration_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Clone_Declaration_Opaque_OList_To_Json
      (Arg : W_Clone_Declaration_Opaque_OList)
      return JSON_Value;

   function W_Clone_Declaration_Opaque_OList_To_Json
      (Arg : W_Clone_Declaration_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Clone_Declaration_Opaque_Id_To_Json
      (Arg : W_Clone_Declaration_Opaque_Id)
      return JSON_Value;

   function W_Clone_Declaration_Opaque_Id_To_Json
      (Arg : W_Clone_Declaration_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Clone_Declaration_Opaque_List_To_Json
      (Arg : W_Clone_Declaration_Opaque_List)
      return JSON_Value;

   function W_Clone_Declaration_Opaque_List_To_Json
      (Arg : W_Clone_Declaration_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Clone_Substitution_Opaque_OId_To_Json
      (Arg : W_Clone_Substitution_Opaque_OId)
      return JSON_Value;

   function W_Clone_Substitution_Opaque_OId_To_Json
      (Arg : W_Clone_Substitution_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Clone_Substitution_Opaque_OList_To_Json
      (Arg : W_Clone_Substitution_Opaque_OList)
      return JSON_Value;

   function W_Clone_Substitution_Opaque_OList_To_Json
      (Arg : W_Clone_Substitution_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Clone_Substitution_Opaque_Id_To_Json
      (Arg : W_Clone_Substitution_Opaque_Id)
      return JSON_Value;

   function W_Clone_Substitution_Opaque_Id_To_Json
      (Arg : W_Clone_Substitution_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Clone_Substitution_Opaque_List_To_Json
      (Arg : W_Clone_Substitution_Opaque_List)
      return JSON_Value;

   function W_Clone_Substitution_Opaque_List_To_Json
      (Arg : W_Clone_Substitution_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Include_Declaration_Opaque_OId_To_Json
      (Arg : W_Include_Declaration_Opaque_OId)
      return JSON_Value;

   function W_Include_Declaration_Opaque_OId_To_Json
      (Arg : W_Include_Declaration_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Include_Declaration_Opaque_OList_To_Json
      (Arg : W_Include_Declaration_Opaque_OList)
      return JSON_Value;

   function W_Include_Declaration_Opaque_OList_To_Json
      (Arg : W_Include_Declaration_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Include_Declaration_Opaque_Id_To_Json
      (Arg : W_Include_Declaration_Opaque_Id)
      return JSON_Value;

   function W_Include_Declaration_Opaque_Id_To_Json
      (Arg : W_Include_Declaration_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Include_Declaration_Opaque_List_To_Json
      (Arg : W_Include_Declaration_Opaque_List)
      return JSON_Value;

   function W_Include_Declaration_Opaque_List_To_Json
      (Arg : W_Include_Declaration_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Theory_Declaration_Opaque_OId_To_Json
      (Arg : W_Theory_Declaration_Opaque_OId)
      return JSON_Value;

   function W_Theory_Declaration_Opaque_OId_To_Json
      (Arg : W_Theory_Declaration_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Theory_Declaration_Opaque_OList_To_Json
      (Arg : W_Theory_Declaration_Opaque_OList)
      return JSON_Value;

   function W_Theory_Declaration_Opaque_OList_To_Json
      (Arg : W_Theory_Declaration_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Theory_Declaration_Opaque_Id_To_Json
      (Arg : W_Theory_Declaration_Opaque_Id)
      return JSON_Value;

   function W_Theory_Declaration_Opaque_Id_To_Json
      (Arg : W_Theory_Declaration_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Theory_Declaration_Opaque_List_To_Json
      (Arg : W_Theory_Declaration_Opaque_List)
      return JSON_Value;

   function W_Theory_Declaration_Opaque_List_To_Json
      (Arg : W_Theory_Declaration_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Module_Opaque_OId_To_Json
      (Arg : W_Module_Opaque_OId)
      return JSON_Value;

   function W_Module_Opaque_OId_To_Json
      (Arg : W_Module_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Module_Opaque_OList_To_Json
      (Arg : W_Module_Opaque_OList)
      return JSON_Value;

   function W_Module_Opaque_OList_To_Json
      (Arg : W_Module_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Module_Opaque_Id_To_Json
      (Arg : W_Module_Opaque_Id)
      return JSON_Value;

   function W_Module_Opaque_Id_To_Json
      (Arg : W_Module_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Module_Opaque_List_To_Json
      (Arg : W_Module_Opaque_List)
      return JSON_Value;

   function W_Module_Opaque_List_To_Json
      (Arg : W_Module_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Expr_Opaque_OId_To_Json
      (Arg : W_Expr_Opaque_OId)
      return JSON_Value;

   function W_Expr_Opaque_OId_To_Json
      (Arg : W_Expr_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Expr_Opaque_OList_To_Json
      (Arg : W_Expr_Opaque_OList)
      return JSON_Value;

   function W_Expr_Opaque_OList_To_Json
      (Arg : W_Expr_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Expr_Opaque_Id_To_Json
      (Arg : W_Expr_Opaque_Id)
      return JSON_Value;

   function W_Expr_Opaque_Id_To_Json
      (Arg : W_Expr_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Expr_Opaque_List_To_Json
      (Arg : W_Expr_Opaque_List)
      return JSON_Value;

   function W_Expr_Opaque_List_To_Json
      (Arg : W_Expr_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Pred_Opaque_OId_To_Json
      (Arg : W_Pred_Opaque_OId)
      return JSON_Value;

   function W_Pred_Opaque_OId_To_Json
      (Arg : W_Pred_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Pred_Opaque_OList_To_Json
      (Arg : W_Pred_Opaque_OList)
      return JSON_Value;

   function W_Pred_Opaque_OList_To_Json
      (Arg : W_Pred_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Pred_Opaque_Id_To_Json
      (Arg : W_Pred_Opaque_Id)
      return JSON_Value;

   function W_Pred_Opaque_Id_To_Json
      (Arg : W_Pred_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Pred_Opaque_List_To_Json
      (Arg : W_Pred_Opaque_List)
      return JSON_Value;

   function W_Pred_Opaque_List_To_Json
      (Arg : W_Pred_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Term_Opaque_OId_To_Json
      (Arg : W_Term_Opaque_OId)
      return JSON_Value;

   function W_Term_Opaque_OId_To_Json
      (Arg : W_Term_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Term_Opaque_OList_To_Json
      (Arg : W_Term_Opaque_OList)
      return JSON_Value;

   function W_Term_Opaque_OList_To_Json
      (Arg : W_Term_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Term_Opaque_Id_To_Json
      (Arg : W_Term_Opaque_Id)
      return JSON_Value;

   function W_Term_Opaque_Id_To_Json
      (Arg : W_Term_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Term_Opaque_List_To_Json
      (Arg : W_Term_Opaque_List)
      return JSON_Value;

   function W_Term_Opaque_List_To_Json
      (Arg : W_Term_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Prog_Opaque_OId_To_Json
      (Arg : W_Prog_Opaque_OId)
      return JSON_Value;

   function W_Prog_Opaque_OId_To_Json
      (Arg : W_Prog_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Prog_Opaque_OList_To_Json
      (Arg : W_Prog_Opaque_OList)
      return JSON_Value;

   function W_Prog_Opaque_OList_To_Json
      (Arg : W_Prog_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Prog_Opaque_Id_To_Json
      (Arg : W_Prog_Opaque_Id)
      return JSON_Value;

   function W_Prog_Opaque_Id_To_Json
      (Arg : W_Prog_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Prog_Opaque_List_To_Json
      (Arg : W_Prog_Opaque_List)
      return JSON_Value;

   function W_Prog_Opaque_List_To_Json
      (Arg : W_Prog_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Type_Definition_Opaque_OId_To_Json
      (Arg : W_Type_Definition_Opaque_OId)
      return JSON_Value;

   function W_Type_Definition_Opaque_OId_To_Json
      (Arg : W_Type_Definition_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Type_Definition_Opaque_OList_To_Json
      (Arg : W_Type_Definition_Opaque_OList)
      return JSON_Value;

   function W_Type_Definition_Opaque_OList_To_Json
      (Arg : W_Type_Definition_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Type_Definition_Opaque_Id_To_Json
      (Arg : W_Type_Definition_Opaque_Id)
      return JSON_Value;

   function W_Type_Definition_Opaque_Id_To_Json
      (Arg : W_Type_Definition_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Type_Definition_Opaque_List_To_Json
      (Arg : W_Type_Definition_Opaque_List)
      return JSON_Value;

   function W_Type_Definition_Opaque_List_To_Json
      (Arg : W_Type_Definition_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Declaration_Opaque_OId_To_Json
      (Arg : W_Declaration_Opaque_OId)
      return JSON_Value;

   function W_Declaration_Opaque_OId_To_Json
      (Arg : W_Declaration_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Declaration_Opaque_OList_To_Json
      (Arg : W_Declaration_Opaque_OList)
      return JSON_Value;

   function W_Declaration_Opaque_OList_To_Json
      (Arg : W_Declaration_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Declaration_Opaque_Id_To_Json
      (Arg : W_Declaration_Opaque_Id)
      return JSON_Value;

   function W_Declaration_Opaque_Id_To_Json
      (Arg : W_Declaration_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Declaration_Opaque_List_To_Json
      (Arg : W_Declaration_Opaque_List)
      return JSON_Value;

   function W_Declaration_Opaque_List_To_Json
      (Arg : W_Declaration_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   function W_Any_Node_Opaque_OId_To_Json
      (Arg : W_Any_Node_Opaque_OId)
      return JSON_Value;

   function W_Any_Node_Opaque_OId_To_Json
      (Arg : W_Any_Node_Opaque_OId)
      return JSON_Value
      is (Why_Node_OId_To_Json (Arg));

   function W_Any_Node_Opaque_OList_To_Json
      (Arg : W_Any_Node_Opaque_OList)
      return JSON_Value;

   function W_Any_Node_Opaque_OList_To_Json
      (Arg : W_Any_Node_Opaque_OList)
      return JSON_Value
      is (Why_Node_OList_To_Json (Arg));

   function W_Any_Node_Opaque_Id_To_Json
      (Arg : W_Any_Node_Opaque_Id)
      return JSON_Value;

   function W_Any_Node_Opaque_Id_To_Json
      (Arg : W_Any_Node_Opaque_Id)
      return JSON_Value
      is (Why_Node_Id_To_Json (Arg));

   function W_Any_Node_Opaque_List_To_Json
      (Arg : W_Any_Node_Opaque_List)
      return JSON_Value;

   function W_Any_Node_Opaque_List_To_Json
      (Arg : W_Any_Node_Opaque_List)
      return JSON_Value
      is (Why_Node_List_To_Json (Arg));

   Why_Node_Counter : Integer := 0;

   function Why_Node_To_Json (Node : Why_Node) return JSON_Value is
      Res : constant JSON_Value := Create (Empty_Array);
   begin
      Why_Node_Counter := Why_Node_Counter + 1;
      Append (Res, Create (Why_Node_Kind'Image (Node.Kind)));
      Append (Res, Create (Why_Node_Counter));
      Append (Res, Node_Id_To_Json (Node.Ada_Node));
      Append (Res, EW_Domain_To_Json (Node.Domain));
      Append (Res, Why_Node_Set_To_Json (Node.Link));
      Append (Res, Boolean_To_Json (Node.Checked));
      case Node.Kind is
         when W_Unused_At_Start =>
            null;
         when W_Type =>
            Append (Res, EW_Type_To_Json
               (Node.K1_Type_Kind));
            Append (Res, W_Name_Opaque_Id_To_Json
               (Node.K1_Name));
            Append (Res, Boolean_To_Json
               (Node.K1_Is_Mutable));
            Append (Res, Boolean_To_Json
               (Node.K1_Relaxed_Init));
         when W_Name =>
            Append (Res, Symbol_To_Json
               (Node.K2_Symb));
            Append (Res, Symbol_To_Json
               (Node.K2_Namespace));
            Append (Res, W_Module_Opaque_OId_To_Json
               (Node.K2_Module));
            Append (Res, Boolean_To_Json
               (Node.K2_Infix));
         when W_Effects =>
            Append (Res, W_Identifier_Opaque_OList_To_Json
               (Node.K3_Reads));
            Append (Res, W_Identifier_Opaque_OList_To_Json
               (Node.K3_Writes));
            Append (Res, W_Raise_Effect_Opaque_OList_To_Json
               (Node.K3_Raises));
         when W_Raise_Effect =>
            Append (Res, W_Name_Opaque_Id_To_Json
               (Node.K4_Name));
            Append (Res, W_Identifier_Opaque_OId_To_Json
               (Node.K4_Arg_Id));
            Append (Res, W_Pred_Opaque_OId_To_Json
               (Node.K4_Post));
         when W_Binder =>
            Append (Res, W_Identifier_Opaque_OId_To_Json
               (Node.K5_Name));
            Append (Res, W_Type_Opaque_Id_To_Json
               (Node.K5_Arg_Type));
         when W_Transparent_Type_Definition =>
            Append (Res, W_Type_Opaque_Id_To_Json
               (Node.K6_Type_Definition));
         when W_Record_Binder =>
            Append (Res, W_Identifier_Opaque_OId_To_Json
               (Node.K7_Name));
            Append (Res, W_Type_Opaque_Id_To_Json
               (Node.K7_Arg_Type));
            Append (Res, Symbol_Set_To_Json
               (Node.K7_Labels));
            Append (Res, Boolean_To_Json
               (Node.K7_Is_Mutable));
         when W_Record_Definition =>
            Append (Res, W_Record_Binder_Opaque_List_To_Json
               (Node.K8_Fields));
         when W_Range_Type_Definition =>
            Append (Res, Uint_To_Json
               (Node.K9_First));
            Append (Res, Uint_To_Json
               (Node.K9_Last));
         when W_Triggers =>
            Append (Res, W_Trigger_Opaque_List_To_Json
               (Node.K10_Triggers));
         when W_Trigger =>
            Append (Res, W_Expr_Opaque_List_To_Json
               (Node.K11_Terms));
         when W_Axiom_Dep =>
            Append (Res, W_Identifier_Opaque_Id_To_Json
               (Node.K12_Name));
            Append (Res, EW_Axiom_Dep_Kind_To_Json
               (Node.K12_Kind));
         when W_Handler =>
            Append (Res, W_Name_Opaque_Id_To_Json
               (Node.K13_Name));
            Append (Res, W_Identifier_Opaque_OId_To_Json
               (Node.K13_Arg_Id));
            Append (Res, W_Prog_Opaque_Id_To_Json
               (Node.K13_Def));
         when W_Field_Association =>
            Append (Res, W_Identifier_Opaque_Id_To_Json
               (Node.K14_Field));
            Append (Res, W_Expr_Opaque_Id_To_Json
               (Node.K14_Value));
         when W_Variant =>
            Append (Res, W_Identifier_Opaque_Id_To_Json
               (Node.K15_Cmp_Op));
            Append (Res, Symbol_Set_To_Json
               (Node.K15_Labels));
            Append (Res, W_Term_Opaque_Id_To_Json
               (Node.K15_Expr));
         when W_Variants =>
            Append (Res, W_Variant_Opaque_List_To_Json
               (Node.K16_Variants));
         when W_Universal_Quantif =>
            Append (Res, W_Identifier_Opaque_List_To_Json
               (Node.K17_Variables));
            Append (Res, Symbol_Set_To_Json
               (Node.K17_Labels));
            Append (Res, W_Type_Opaque_Id_To_Json
               (Node.K17_Var_Type));
            Append (Res, W_Triggers_Opaque_OId_To_Json
               (Node.K17_Triggers));
            Append (Res, W_Pred_Opaque_Id_To_Json
               (Node.K17_Pred));
         when W_Existential_Quantif =>
            Append (Res, W_Identifier_Opaque_List_To_Json
               (Node.K18_Variables));
            Append (Res, Symbol_Set_To_Json
               (Node.K18_Labels));
            Append (Res, W_Type_Opaque_Id_To_Json
               (Node.K18_Var_Type));
            Append (Res, W_Pred_Opaque_Id_To_Json
               (Node.K18_Pred));
         when W_Not =>
            Append (Res, W_Expr_Opaque_Id_To_Json
               (Node.K19_Right));
         when W_Connection =>
            Append (Res, W_Expr_Opaque_Id_To_Json
               (Node.K20_Left));
            Append (Res, EW_Connector_To_Json
               (Node.K20_Op));
            Append (Res, W_Expr_Opaque_Id_To_Json
               (Node.K20_Right));
            Append (Res, W_Expr_Opaque_OList_To_Json
               (Node.K20_More_Right));
         when W_Label =>
            Append (Res, Symbol_Set_To_Json
               (Node.K21_Labels));
            Append (Res, W_Expr_Opaque_Id_To_Json
               (Node.K21_Def));
            Append (Res, W_Type_Opaque_OId_To_Json
               (Node.K21_Typ));
         when W_Loc_Label =>
            Append (Res, Source_Ptr_To_Json
               (Node.K22_Sloc));
            Append (Res, W_Expr_Opaque_Id_To_Json
               (Node.K22_Def));
            Append (Res, Symbol_To_Json
               (Node.K22_Marker));
         when W_Identifier =>
            Append (Res, W_Name_Opaque_Id_To_Json
               (Node.K23_Name));
            Append (Res, W_Type_Opaque_OId_To_Json
               (Node.K23_Typ));
            Append (Res, String_Sets_Set_To_Json
               (Node.K23_Labels));
         when W_Tagged =>
            Append (Res, Symbol_To_Json
               (Node.K24_Tag));
            Append (Res, W_Expr_Opaque_Id_To_Json
               (Node.K24_Def));
            Append (Res, W_Type_Opaque_OId_To_Json
               (Node.K24_Typ));
         when W_Call =>
            Append (Res, W_Identifier_Opaque_Id_To_Json
               (Node.K25_Name));
            Append (Res, W_Expr_Opaque_OList_To_Json
               (Node.K25_Args));
            Append (Res, W_Type_Opaque_OId_To_Json
               (Node.K25_Typ));
         when W_Literal =>
            Append (Res, EW_Literal_To_Json
               (Node.K26_Value));
            Append (Res, W_Type_Opaque_OId_To_Json
               (Node.K26_Typ));
         when W_Binding =>
            Append (Res, W_Identifier_Opaque_Id_To_Json
               (Node.K27_Name));
            Append (Res, W_Expr_Opaque_Id_To_Json
               (Node.K27_Def));
            Append (Res, W_Expr_Opaque_Id_To_Json
               (Node.K27_Context));
            Append (Res, W_Type_Opaque_OId_To_Json
               (Node.K27_Typ));
         when W_Elsif =>
            Append (Res, W_Expr_Opaque_Id_To_Json
               (Node.K28_Condition));
            Append (Res, W_Expr_Opaque_Id_To_Json
               (Node.K28_Then_Part));
            Append (Res, W_Type_Opaque_OId_To_Json
               (Node.K28_Typ));
         when W_Epsilon =>
            Append (Res, W_Identifier_Opaque_Id_To_Json
               (Node.K29_Name));
            Append (Res, W_Type_Opaque_Id_To_Json
               (Node.K29_Typ));
            Append (Res, W_Pred_Opaque_Id_To_Json
               (Node.K29_Pred));
         when W_Conditional =>
            Append (Res, W_Expr_Opaque_Id_To_Json
               (Node.K30_Condition));
            Append (Res, W_Expr_Opaque_Id_To_Json
               (Node.K30_Then_Part));
            Append (Res, W_Expr_Opaque_OList_To_Json
               (Node.K30_Elsif_Parts));
            Append (Res, W_Expr_Opaque_OId_To_Json
               (Node.K30_Else_Part));
            Append (Res, W_Type_Opaque_OId_To_Json
               (Node.K30_Typ));
         when W_Integer_Constant =>
            Append (Res, Uint_To_Json
               (Node.K31_Value));
         when W_Range_Constant =>
            Append (Res, Uint_To_Json
               (Node.K32_Value));
            Append (Res, W_Type_Opaque_Id_To_Json
               (Node.K32_Typ));
         when W_Modular_Constant =>
            Append (Res, Uint_To_Json
               (Node.K33_Value));
            Append (Res, W_Type_Opaque_Id_To_Json
               (Node.K33_Typ));
         when W_Fixed_Constant =>
            Append (Res, Uint_To_Json
               (Node.K34_Value));
            Append (Res, W_Type_Opaque_Id_To_Json
               (Node.K34_Typ));
         when W_Real_Constant =>
            Append (Res, Ureal_To_Json
               (Node.K35_Value));
         when W_Float_Constant =>
            Append (Res, Ureal_To_Json
               (Node.K36_Value));
            Append (Res, W_Type_Opaque_Id_To_Json
               (Node.K36_Typ));
         when W_Comment =>
            Append (Res, Symbol_To_Json
               (Node.K37_Comment));
         when W_Deref =>
            Append (Res, W_Identifier_Opaque_Id_To_Json
               (Node.K38_Right));
            Append (Res, W_Type_Opaque_Id_To_Json
               (Node.K38_Typ));
         when W_Record_Access =>
            Append (Res, W_Expr_Opaque_Id_To_Json
               (Node.K39_Name));
            Append (Res, W_Identifier_Opaque_Id_To_Json
               (Node.K39_Field));
            Append (Res, W_Type_Opaque_OId_To_Json
               (Node.K39_Typ));
         when W_Record_Update =>
            Append (Res, W_Expr_Opaque_Id_To_Json
               (Node.K40_Name));
            Append (Res, W_Field_Association_Opaque_List_To_Json
               (Node.K40_Updates));
            Append (Res, W_Type_Opaque_OId_To_Json
               (Node.K40_Typ));
         when W_Record_Aggregate =>
            Append (Res, W_Field_Association_Opaque_List_To_Json
               (Node.K41_Associations));
            Append (Res, W_Type_Opaque_OId_To_Json
               (Node.K41_Typ));
         when W_Any_Expr =>
            Append (Res, W_Effects_Opaque_OId_To_Json
               (Node.K42_Effects));
            Append (Res, W_Pred_Opaque_OId_To_Json
               (Node.K42_Pre));
            Append (Res, W_Pred_Opaque_OId_To_Json
               (Node.K42_Post));
            Append (Res, W_Type_Opaque_Id_To_Json
               (Node.K42_Return_Type));
            Append (Res, Symbol_Set_To_Json
               (Node.K42_Labels));
         when W_Assignment =>
            Append (Res, W_Identifier_Opaque_Id_To_Json
               (Node.K43_Name));
            Append (Res, W_Prog_Opaque_Id_To_Json
               (Node.K43_Value));
            Append (Res, W_Type_Opaque_Id_To_Json
               (Node.K43_Typ));
            Append (Res, Symbol_Set_To_Json
               (Node.K43_Labels));
         when W_Binding_Ref =>
            Append (Res, W_Identifier_Opaque_Id_To_Json
               (Node.K44_Name));
            Append (Res, W_Prog_Opaque_Id_To_Json
               (Node.K44_Def));
            Append (Res, W_Prog_Opaque_Id_To_Json
               (Node.K44_Context));
            Append (Res, W_Type_Opaque_Id_To_Json
               (Node.K44_Typ));
         when W_Loop =>
            Append (Res, W_Prog_Opaque_Id_To_Json
               (Node.K45_Code_Before));
            Append (Res, W_Pred_Opaque_OList_To_Json
               (Node.K45_Invariants));
            Append (Res, W_Variants_Opaque_OList_To_Json
               (Node.K45_Variants));
            Append (Res, W_Prog_Opaque_Id_To_Json
               (Node.K45_Code_After));
         when W_Statement_Sequence =>
            Append (Res, W_Prog_Opaque_List_To_Json
               (Node.K46_Statements));
         when W_Abstract_Expr =>
            Append (Res, W_Prog_Opaque_Id_To_Json
               (Node.K47_Expr));
            Append (Res, W_Pred_Opaque_Id_To_Json
               (Node.K47_Post));
            Append (Res, W_Type_Opaque_OId_To_Json
               (Node.K47_Typ));
         when W_Assert =>
            Append (Res, W_Pred_Opaque_Id_To_Json
               (Node.K48_Pred));
            Append (Res, EW_Assert_Kind_To_Json
               (Node.K48_Assert_Kind));
         when W_Raise =>
            Append (Res, W_Name_Opaque_Id_To_Json
               (Node.K49_Name));
            Append (Res, W_Expr_Opaque_OId_To_Json
               (Node.K49_Arg));
            Append (Res, W_Type_Opaque_OId_To_Json
               (Node.K49_Typ));
         when W_Try_Block =>
            Append (Res, W_Prog_Opaque_Id_To_Json
               (Node.K50_Prog));
            Append (Res, W_Handler_Opaque_List_To_Json
               (Node.K50_Handler));
            Append (Res, W_Type_Opaque_OId_To_Json
               (Node.K50_Typ));
         when W_Function_Decl =>
            Append (Res, W_Identifier_Opaque_Id_To_Json
               (Node.K51_Name));
            Append (Res, W_Binder_Opaque_OList_To_Json
               (Node.K51_Binders));
            Append (Res, W_Effects_Opaque_OId_To_Json
               (Node.K51_Effects));
            Append (Res, W_Pred_Opaque_OId_To_Json
               (Node.K51_Pre));
            Append (Res, W_Pred_Opaque_OId_To_Json
               (Node.K51_Post));
            Append (Res, W_Type_Opaque_OId_To_Json
               (Node.K51_Return_Type));
            Append (Res, W_Expr_Opaque_OId_To_Json
               (Node.K51_Def));
            Append (Res, Symbol_Set_To_Json
               (Node.K51_Labels));
            Append (Res, Source_Ptr_To_Json
               (Node.K51_Location));
         when W_Axiom =>
            Append (Res, Symbol_To_Json
               (Node.K52_Name));
            Append (Res, W_Pred_Opaque_Id_To_Json
               (Node.K52_Def));
            Append (Res, W_Axiom_Dep_Opaque_OId_To_Json
               (Node.K52_Dep));
         when W_Goal =>
            Append (Res, Symbol_To_Json
               (Node.K53_Name));
            Append (Res, W_Pred_Opaque_Id_To_Json
               (Node.K53_Def));
         when W_Type_Decl =>
            Append (Res, W_Identifier_Opaque_OList_To_Json
               (Node.K54_Args));
            Append (Res, W_Name_Opaque_Id_To_Json
               (Node.K54_Name));
            Append (Res, Symbol_Set_To_Json
               (Node.K54_Labels));
            Append (Res, W_Type_Definition_Opaque_OId_To_Json
               (Node.K54_Definition));
         when W_Global_Ref_Declaration =>
            Append (Res, W_Identifier_Opaque_Id_To_Json
               (Node.K55_Name));
            Append (Res, W_Type_Opaque_Id_To_Json
               (Node.K55_Ref_Type));
            Append (Res, Symbol_Set_To_Json
               (Node.K55_Labels));
            Append (Res, Source_Ptr_To_Json
               (Node.K55_Location));
         when W_Namespace_Declaration =>
            Append (Res, W_Declaration_Opaque_OList_To_Json
               (Node.K56_Declarations));
            Append (Res, Symbol_To_Json
               (Node.K56_Name));
         when W_Exception_Declaration =>
            Append (Res, W_Name_Opaque_Id_To_Json
               (Node.K57_Name));
            Append (Res, W_Type_Opaque_OId_To_Json
               (Node.K57_Arg));
         when W_Meta_Declaration =>
            Append (Res, Symbol_To_Json
               (Node.K58_Name));
            Append (Res, Symbol_To_Json
               (Node.K58_Parameter));
         when W_Clone_Declaration =>
            Append (Res, W_Module_Opaque_Id_To_Json
               (Node.K59_Origin));
            Append (Res, Symbol_To_Json
               (Node.K59_As_Name));
            Append (Res, EW_Clone_Type_To_Json
               (Node.K59_Clone_Kind));
            Append (Res, W_Clone_Substitution_Opaque_OList_To_Json
               (Node.K59_Substitutions));
            Append (Res, EW_Theory_Type_To_Json
               (Node.K59_Theory_Kind));
         when W_Clone_Substitution =>
            Append (Res, EW_Subst_Type_To_Json
               (Node.K60_Kind));
            Append (Res, W_Name_Opaque_Id_To_Json
               (Node.K60_Orig_Name));
            Append (Res, W_Name_Opaque_Id_To_Json
               (Node.K60_Image));
         when W_Include_Declaration =>
            Append (Res, W_Module_Opaque_Id_To_Json
               (Node.K61_Module));
            Append (Res, EW_Theory_Type_To_Json
               (Node.K61_Kind));
            Append (Res, EW_Clone_Type_To_Json
               (Node.K61_Use_Kind));
         when W_Theory_Declaration =>
            Append (Res, W_Declaration_Opaque_OList_To_Json
               (Node.K62_Declarations));
            Append (Res, Symbol_To_Json
               (Node.K62_Name));
            Append (Res, EW_Theory_Type_To_Json
               (Node.K62_Kind));
            Append (Res, W_Include_Declaration_Opaque_OList_To_Json
               (Node.K62_Includes));
            Append (Res, Symbol_To_Json
               (Node.K62_Comment));
         when W_Module =>
            Append (Res, Symbol_To_Json
               (Node.K63_File));
            Append (Res, Symbol_To_Json
               (Node.K63_Name));
      end case;
      return Res;
   end Why_Node_To_Json;
end Why.Atree.To_Json;